# copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@free.fr)
# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu

set rcsId {$Id: print.tcl,v 1.6.1.1 1999/12/30 22:07:57 jfontain Exp $}


namespace eval print {

    variable dotsPerMillimeter [expr {72/25.4}]                                          ;# 72 dots/inch is the default for printers
    variable helpMessage \
{  Print to a printer device or a file in Postscript.
  The print command typically reads the Postscript data
from its standard input and redirects it to the specified
printer.
  For example, on UNIX, 'lpr -Pacme' will print the
canvas area on the acme printer, whereas 'gv -' will
launch ghostview for previewing.}

    proc printOrSaveCanvas {} {
        variable printToFile $global::printToFile
        variable printCommand $global::printCommand
        variable fileToPrintTo $global::fileToPrintTo
        variable printOrientation $global::printOrientation
        variable printColorMode $global::printColorMode
        variable printPaperSize $global::printPaperSize
        variable helpMessage

        set dialog\
            [new dialogBox . -buttons oc -default o -title {moodss: Print} -die 0 -x [winfo pointerx .] -y [winfo pointery .]]
        set frame [frame $widget::($dialog,path).frame]

        set row 0
        if {[catch {$global::canvas postscript -colormode mono -width 1 -height 1 -prolog 0}]} {
            message $frame.warning -width [winfo screenwidth .] -font $font::(mediumBold) -justify left -text \
{WARNING: your Tcl/Tk interpreter does not seem
to have the Dash patch installed
(tk8.0.5dash.patch.gz at ftp.neosoft.com in the
/pub/tcl/sorted/packages-8.0/devel/ directory)
and you are likely to obtain a blank page as printout.}
            grid $frame.warning -pady 5 -row $row -column 0 -columnspan 100
            incr row
        }
        message $frame.help -width [winfo screenwidth .] -font $font::(mediumNormal) -justify left -text $helpMessage
        grid $frame.help -pady 5 -row $row -column 0 -columnspan 100

        incr row
        radiobutton $frame.toCommand -variable print::printToFile -value 0 -text {with Command:}
        grid $frame.toCommand -row $row -column 0 -sticky w
        entry $frame.command -textvariable print::printCommand
        grid $frame.command -row $row -column 1 -columnspan 100 -sticky ew
        incr row
        radiobutton $frame.toFile -variable print::printToFile -value 1 -text {or to File:}
        grid $frame.toFile -row $row -column 0 -sticky w
        entry $frame.file -textvariable print::fileToPrintTo
        grid $frame.file -row $row -column 1 -sticky ew
        button $frame.browse -text Browse... -command "inquireFileToPrintTo $frame"
        grid $frame.browse -row $row -column 2
        $frame.toCommand configure -command "print::setEntriesState 0 $frame.command $frame.file $frame.browse"
        $frame.toFile configure -command "print::setEntriesState 1 $frame.command $frame.file $frame.browse"
        if {$printToFile} {
            $frame.toFile invoke
        } else {
            $frame.toCommand invoke
        }

        incr row
        grid [label $frame.orientation -text Orientation:] -row $row -column 0 -sticky w
        set orientation [new comboEntry $frame -font $widget::(default,EntryFont) -list {landscape portrait} -editable 0]
        composite::configure $orientation entry -textvariable print::printOrientation
        composite::configure $orientation button -listheight 2
        composite::configure $orientation button scroll -selectmode single
        grid $widget::($orientation,path) -row $row -column 1 -columnspan 100 -sticky ew

        incr row
        grid [label $frame.colorMode -text Mode:] -row $row -column 0 -sticky w
        set colorMode [new comboEntry $frame -font $widget::(default,EntryFont) -list {color gray monochrome} -editable 0]
        composite::configure $colorMode entry -textvariable print::printColorMode
        composite::configure $colorMode button scroll -selectmode single
        grid $widget::($colorMode,path) -row $row -column 1 -columnspan 100 -sticky ew

        incr row
        grid [label $frame.paperSize -text {Paper size:}] -row $row -column 0 -sticky w
        set paperSize [new comboEntry $frame -font $widget::(default,EntryFont) -list $global::printPaperSizes -editable 0]
        composite::configure $paperSize entry -textvariable print::printPaperSize
        composite::configure $paperSize button -listheight 5
        composite::configure $paperSize button scroll -selectmode single
        grid $widget::($paperSize,path) -row $row -column 1 -columnspan 100 -sticky ew

        dialogBox::display $dialog $frame
        # make sure dialog box does not appear in printout
        widget::configure $dialog -command "delete $dialog; print::updateGlobals; update; print::print"
        bind $frame <Destroy> "delete $orientation $colorMode $paperSize"            ;# delete objects not managed by the dialog box
    }

    proc setEntriesState {toFile command file browse} {
        if {$toFile} {
            $command configure -state disabled -foreground $widget::(default,ButtonDisabledForegroundColor)
            $file configure -state normal -foreground $widget::(default,ButtonForeground)
            $browse configure -state normal
        } else {
            $command configure -state normal -foreground $widget::(default,ButtonForeground)
            $file configure -state disabled -foreground $widget::(default,ButtonDisabledForegroundColor)
            $browse configure -state disabled
        }
    }

    proc inquireFileToPrintTo {parentPath} {
        set file [tk_getSaveFile\
            -title {moodss: Print to file} -parent $parentPath -initialdir [pwd] -defaultextension .ps\
            -filetypes {{Postscript .ps} {{All files} *}} -initialfile $print::fileToPrintTo\
        ]
        if {[string length $file]>0} {                                                                              ;# not cancelled
            set print::fileToPrintTo $file
        }
    }

    proc updateGlobals {} {
        variable printToFile
        variable printCommand
        variable fileToPrintTo
        variable printOrientation
        variable printColorMode
        variable printPaperSize

        set global::printToFile $printToFile
        set global::printCommand $printCommand
        set global::fileToPrintTo $fileToPrintTo
        set global::printOrientation $printOrientation
        set global::printColorMode $printColorMode
        set global::printPaperSize $printPaperSize
    }

    proc print {} {
        variable printOrientation
        variable printPaperSize
        variable printToFile
        variable fileToPrintTo
        variable printColorMode
        variable printCommand
        variable dotsPerMillimeter

        update                                          ;# make sure everything is redrawn properly in case dialog box hid something

        set canvas $global::canvas
        # first calculate visible area coordinates
        foreach {left top right bottom} [$canvas cget -scrollregion] {}
        set width [expr {$right-$left}]
        set height [expr {$bottom-$top}]
        foreach {minimum maximum} [$canvas xview] {}
        set left [expr {$left+($minimum*$width)}]
        foreach {minimum maximum} [$canvas yview] {}
        set top [expr {$top+($minimum*$height)}]
        scan [winfo geometry $canvas] %ux%u width height                                                ;# now get visible area size
        set right [expr {$left+$width}]
        set bottom [expr {$top+$height}]
        # the eventually minimize to visible items
        foreach {boundsLeft boundsTop boundsRight boundsBottom} [$canvas bbox all] {}
        if {$boundsLeft>$left} {set left $boundsLeft}
        if {$boundsRight<$right} {set right $boundsRight}
        if {$boundsTop>$top} {set top $boundsTop}
        if {$boundsBottom<$bottom} {set bottom $boundsBottom}
        set width [expr {$right-$left}]
        set height [expr {$bottom-$top}]
        set inch 25.4                                                                                                 ;# millimeters
        set margin [expr {$inch/2}]                                                                 ;# all dimensions in millimeters
        switch -glob $printPaperSize {
            A3* {
                set pageWidth 297
                set pageHeight 420
            }
            A4* {
                set pageWidth 210
                set pageHeight 297
            }
            executive* {
                set pageWidth [expr {7.5*$inch}]
                set pageHeight [expr {10*$inch}]
            }
            legal* {
                set pageWidth [expr {8.5*$inch}]
                set pageHeight [expr {14*$inch}]
            }
            default {
                set pageWidth [expr {8.5*$inch}]                                                                           ;# letter
                set pageHeight [expr {11*$inch}]
            }
        }
        set pageX ${margin}m
        set rotate [expr {[string compare $printOrientation landscape]==0}]
        if {$rotate} {
            set pageY ${margin}m
        } else {
            set pageY [expr {$pageHeight-$margin}]m
        }
        # now make sure everything fits in page
        if {$rotate} {                                                                                 ;# swap page width and height
            set value $pageWidth
            set pageWidth $pageHeight
            set pageHeight $value
            unset value
        }
        set printWidth [expr {($pageWidth-(2*$margin))*$dotsPerMillimeter}]                                             ;# in pixels
        set printHeight [expr {($pageHeight-(2*$margin))*$dotsPerMillimeter}]
        set ratio 1                                                                  ;# use identical horizontal and vertical ratios
        if {$printWidth<$width} {
            set ratio [expr {$printWidth/$width}]
        }
        if {($printHeight<$height)&&(($printHeight/$height)<$ratio)} {
            set ratio [expr {$printHeight/$height}]
        }
        set options [list\
            -colormode $printColorMode -rotate $rotate -x $left -y $top -width $width -height $height\
            -pageanchor nw -pagex $pageX -pagey $pageY\
        ]
        if {$ratio<1} {                                                                                     ;# size reduction needed
            lappend options -pagewidth [expr {$ratio*$width}] -pageheight [expr {$ratio*$height}]
        }
        # finally print
        if {$printToFile} {
            lifoLabel::push $global::messenger "printing to file $fileToPrintTo..."
        } else {
            lifoLabel::push $global::messenger printing...
        }
        update idletasks                                                                             ;# make sure message is visible
        if {$printToFile} {
            lappend options -file $fileToPrintTo
            eval $canvas postscript $options
        } else {                             ;# gather data first in case window gets obscured by printing utility (which may be gv)
            set data [eval $canvas postscript $options]
            set channel [open |$printCommand w]
            puts -nonewline $channel $data
            close $channel
        }
        lifoLabel::pop $global::messenger
    }

}
